/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 *  Authors: Iain Holmes <iain@prettypeople.org>
 *
 *  Copyright 2002 Iain Holmes
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public 
 * License as published by the Free Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef __MARLIN_CHANNEL_H__
#define __MARLIN_CHANNEL_H__

#include <glib.h>

#include <marlin/marlin-types.h>
#include <marlin/marlin-read-write-lock.h>
#include <marlin/marlin-operation.h>
#include <marlin/marlin-undo-manager.h>

#define MARLIN_CHANNEL_ERROR marlin_channel_error_quark ()

typedef struct _MarlinPeak {
	float low;
	float high;
	float avg_low;
	float avg_high;
} MarlinPeak;

struct _FileRegion {
	off_t offset;
	size_t length;
	guint64 frames;
};

typedef enum {
	MARLIN_CHANNEL_ERROR_IO, /* Something failed while disk IO...*/
	MARLIN_CHANNEL_ERROR_NO_SPACE, /* Disk IO failed with no space */
	MARLIN_CHANNEL_ERROR_MEMORY, /* MMap failed */
} MarlinChannelError;

struct _MarlinChannel {
	MarlinReadWriteLock *lock; /* The lock on the data:
				      Only access first and last when you
				      hold this lock */
	MarlinSample *sample; /* The sample a channel is part of */

	gboolean mmapped; /* Is all the data mmapped? */
	struct _FileRegion file_region;
	
	char *mmap_file; /* name of the mmap file */
	int mmap_fd; /* file descriptor for mmap file */

	char *peak_file; /* Name of the mmap file for the peaks */
	int peak_fd; /* File descriptor for mmap peak file */
	
	GList *mmap_regions; /* A list of pointers to the mmaped regions */
	
	MarlinBlock *first, *last; /* We store the first and last blocks so
				      that append is a O(1) operation. */
	MarlinBlock *peak_first, *peak_last; /* These are the peak blocks */
	
	guint64 frames;
};

MarlinChannel *marlin_channel_new (MarlinSample *sample,
				   const char *filename,
				   GError **error);
void marlin_channel_free (MarlinChannel *channel);

off_t marlin_channel_write_data (MarlinChannel *channel,
				 float *data,
				 guint num_frames,
				 GError **error);
gboolean marlin_channel_ensure_mmapped (MarlinChannel *channel,
					MarlinOperation *operation,
					GError **error);

MarlinBlock *marlin_channel_get_block_for_frame (MarlinChannel *channel,
						 guint64 frame);
MarlinBlock *marlin_channel_get_peak_for_frame (MarlinChannel *channel,
						guint64 frame);
gboolean marlin_channel_generate_peaks (MarlinChannel *channel,
					MarlinOperation *operation,
					GError **error);
gboolean marlin_channel_insert_data (MarlinChannel *channel,
				     float *data,
				     guint64 num_frames,
				     guint64 insert_frame,
				     MarlinUndoContext *ctxt,
				     GError **error);
gboolean marlin_channel_insert_block (MarlinChannel *channel,
				      MarlinBlock *block,
				      guint64 insert_frame,
				      MarlinUndoContext *ctxt,
				      GError **error);

/* Channel manipulation functions */
void marlin_channel_split_block (MarlinChannel *channel,
				 guint64 split_frame);
void marlin_channel_delete_range (MarlinChannel *channel,
				  guint64 start,
				  guint64 finish,
				  MarlinUndoContext *ctxt);
gboolean marlin_channel_insert_silence (MarlinChannel *channel,
					guint64 insert_frame,
					guint64 num_frames,
					MarlinUndoContext *ctxt,
					GError **error);
gboolean marlin_channel_clear_range (MarlinChannel *channel,
				     guint64 start_frame,
				     guint64 finish_frame,
				     MarlinUndoContext *ctxt,
				     GError **error);
void marlin_channel_crop_range (MarlinChannel *channel,
				guint64 start_frame,
				guint64 finish_frame,
				MarlinUndoContext *ctxt);
gboolean marlin_channel_copy_data (MarlinChannel *src_channel,
				   MarlinChannel *dest_channel,
				   guint64 start_frame,
				   guint64 finish_frame,
				   GError **error);
gboolean marlin_channel_mix (MarlinChannel *dest,
			     MarlinChannel *src,
			     double src_db,
			     double dest_db,
			     guint64 start_frame,
			     guint64 end_frame,
			     gboolean clip,
			     MarlinOperation *operation,
			     MarlinUndoContext *ctxt,
			     GError **error);

gboolean marlin_channel_insert (MarlinChannel *dest,
				MarlinChannel *src,
				guint64 start_frame,
				MarlinUndoContext *ctxt,
				GError **error);

gboolean marlin_channel_invert (MarlinChannel *channel,
				guint64 start,
				guint64 finish,
				MarlinUndoContext *ctxt,
				GError **error);
gboolean marlin_channel_adjust_volume (MarlinChannel *channel,
				       float db,
				       guint64 start,
				       guint64 finish,
				       MarlinUndoContext *ctxt,
				       GError **error);

gboolean marlin_channel_is_ready (MarlinChannel *channel);
#endif
